@@ -3,6 +3,7 @@ import jwt
 import time
 import uuid
 from py4web import request, Session
+from pydal.validators import Validator
 from yatl.helpers import (
     A,
     TEXTAREA,
@@ -19,6 +20,19 @@ from yatl.helpers import (
     XML,
 )
 
+def get_options(validators):
+    options = None
+    if validators:
+        if not isinstance(validators, (list, tuple)):
+            validators = [validators]
+        for item in validators:
+            if hasattr(item, 'options'):
+                options = item.options
+                break
+        if callable(options):
+            options = options()
+    return options
+
 
 class FormStyleFactory:
     def __init__(self):
@@ -46,7 +60,7 @@ class FormStyleFactory:
     def produce(self, table, vars, errors, readonly, deletable, classes=None):
         self.classes.update(classes or {})
         form = FORM(
-            _method="POST", _action=request.path, _enctype="multipart/form-data"
+            _method="POST", _action=request.url, _enctype="multipart/form-data"
         )
         controls = dict(
             widgets=dict(),
@@ -115,16 +129,16 @@ class FormStyleFactory:
                         )
                     )
                     control.append("(check to remove)")
-            elif hasattr(field.requires, "options"):
+            elif get_options(field.requires) is not None:
                 multiple = field.type.startswith("list:")
                 value = list(map(str, value if isinstance(value, list) else [value]))
-                options = [
+                option_tags = [
                     OPTION(v, _value=k, _selected=(not k is None and k in value))
-                    for k, v in field.requires.options()
-                ]
+                    for k, v in get_options(field.requires)
+                    ]
                 control = SELECT(
-                    *options, _id=input_id, _name=field.name, _multiple=multiple
-                )
+                    *option_tags, _id=input_id, _name=field.name, _multiple=multiple
+                     )
             else:
                 field_type = "password" if field.type == "password" else "text"
                 control = INPUT(
@@ -153,7 +167,8 @@ class FormStyleFactory:
                     _class=class_outer,
                 )
             )
-
+            if 'id' in vars:
+                form.append(INPUT(_name='id',_value=vars['id'],_hidden=True))
         if deletable:
             controls["delete"] = INPUT(
                 _type="checkbox",
@@ -285,11 +300,13 @@ class Form(object):
         self.csrf_session = csrf_session
         self.lifespan = lifespan
         self.signing_info = signing_info
+        self.action = None
 
         if readonly or request.method == "GET":
-            self._read_vars_from_record(table)
+            if self.record:
+                self.vars = self._read_vars_from_record(table)
         else:
-            post_vars = request.forms
+            post_vars = self.vars = request.forms
             self.submitted = True
             process = False
 
@@ -300,11 +317,14 @@ class Form(object):
                     process = True
             if process:
                 if not post_vars.get("_delete"):
+                    validated_vars = {}
                     for field in self.table:
                         if field.writable and field.readable:
-                            value = post_vars.get(field.name)
+                            original_value = post_vars.get(field.name)
                             record_id = self.record and self.record.get("id")
-                            (value, error) = field.validate(value, record_id)
+                            (value, error) = field.validate(original_value, record_id)
+                            if field.type == 'password' and record_id and value is None:
+                                continue
                             if field.type == "upload":
                                 value = request.files.get(field.name)
                                 delete = post_vars.get("_delete_" + field.name)
@@ -316,7 +336,7 @@ class Form(object):
                                     value = self.record.get(field.name)
                                 else:
                                     value = None
-                            self.vars[field.name] = value
+                            validated_vars[field.name] = value
                             if error:
                                 self.errors[field.name] = error
                     if validation:
@@ -326,25 +346,24 @@ class Form(object):
                     if not self.errors:
                         self.accepted = True
                         if dbio:
-                            self.update_or_insert()
+                            self.update_or_insert(validated_vars)
                 elif dbio:
                     self.deleted = True
                     self.record.delete_record()
-            else:
+            elif self.record:
                 # This form should not be processed.  We return the same as for GET.
-                self._read_vars_from_record(table)
+                self.vars = self._read_vars_from_record(table)
         self._sign_form()
 
     def _read_vars_from_record(self, table):
-        if self.record:
-            if isinstance(table, list):
-                # The table is just a list of fields.
-                self.vars = {field.name: self.record.get(field.name) for field in table}
-            else:
-                self.vars = {
-                    name: table[name].formatter(self.record[name])
-                    for name in table.fields
-                    if name in self.record
+        if isinstance(table, list):
+            # The table is just a list of fields.
+            return {field.name: self.record.get(field.name) for field in table}
+        else:
+            return {
+                name: table[name].formatter(self.record[name])
+                for name in table.fields
+                if name in self.record
                 }
 
     def _get_key(self):
@@ -384,26 +403,29 @@ class Form(object):
         except:
             return False
 
-    def update_or_insert(self):
+    def update_or_insert(self, validated_vars):
         if self.record:
-            self.record.update_record(**self.vars)
+            self.record.update_record(**validated_vars)
         else:
             # warning, should we really insert if record
-            self.vars["id"] = self.table.insert(**self.vars)
+            self.vars["id"] = self.table.insert(**validated_vars)
 
     def clear(self):
-        self.vars.clear()
         self.errors.clear()
-        for field in self.table:
-            self.vars[field.name] = field.default
+        if not self.record and not self.keep_values:
+            self.vars.clear()
+            for field in self.table:
+                self.vars[field.name] = field.default
 
     def helper(self):
-        if self.accepted and not self.keep_values:
-            self.vars.clear()
+        if self.accepted:
+            self.clear()
         if not self.cached_helper:
             helper = self.formstyle(
                 self.table, self.vars, self.errors, self.readonly, self.deletable
             )
+            if self.action:
+                helper['_action'] = self.action
             if self.form_name:
                 helper["controls"]["hidden_widgets"]["formname"] = INPUT(
                     _type="hidden", _name="_formname", _value=self.form_name
